@@ -2,7 +2,7 @@
 #
 # This file is part of the python-chess library.
 # Copyright (C) 2015 Jean-Noël Avila <jn.avila@free.fr>
-# Copyright (C) 2015-2018 Niklas Fiekas <niklas.fiekas@backscattering.de>
+# Copyright (C) 2015-2019 Niklas Fiekas <niklas.fiekas@backscattering.de>
 #
 # This program is free software: you can redistribute it and/or modify
 # it under the terms of the GNU General Public License as published by
@@ -1530,7 +1530,10 @@ class PythonTablebase:
         self.block_age = 0
 
     def add_directory(self, directory):
-        """Loads *.gtb.cp4* tables from a directory."""
+        """
+        Adds *.gtb.cp4* tables from a directory. The relevant files are lazily
+        opened when the tablebase is actually probed.
+        """
         directory = os.path.abspath(directory)
         if not os.path.isdir(directory):
             raise IOError("not a directory: {}".format(repr(directory)))
@@ -1538,9 +1541,6 @@ class PythonTablebase:
         for tbfile in fnmatch.filter(os.listdir(directory), "*.gtb.cp4"):
             self.available_tables[os.path.basename(tbfile).replace(".gtb.cp4", "")] = os.path.join(directory, tbfile)
 
-    # TODO: Deprecated
-    open_directory = add_directory
-
     def probe_dtm(self, board):
         """
         Probes for depth to mate information.
@@ -1564,6 +1564,8 @@ class PythonTablebase:
             :exc:`chess.gaviota.MissingTableError`) if the probe fails. Use
             :func:`~chess.gaviota.PythonTablebase.get_dtm()` if you prefer
             to get ``None`` instead of an exception.
+
+            Note that probing a corrupted table file is undefined behavior.
         """
         # Can not probe positions with castling rights.
         if board.castling_rights:
@@ -1644,6 +1646,8 @@ class PythonTablebase:
             :exc:`chess.gaviota.MissingTableError`) if the probe fails. Use
             :func:`~chess.gaviota.PythonTablebase.get_wdl()` if you prefer
             to get ``None`` instead of an exception.
+
+            Note that probing a corrupted table file is undefined behavior.
         """
         dtm = self.probe_dtm(board)
 
@@ -1938,9 +1942,6 @@ class NativeTablebase:
         self.paths.append(directory)
         self._tb_restart()
 
-    # TODO: Deprecated
-    open_directory = add_directory
-
     def _tb_restart(self):
         self.c_paths = (ctypes.c_char_p * len(self.paths))()
         self.c_paths[:] = [path.encode("utf-8") for path in self.paths]
@@ -2094,7 +2095,7 @@ def open_tablebase(directory, *, libgtb=None, LibraryLoader=ctypes.cdll):
     The shared library has global state and caches, so only one instance can
     be open at a time.
 
-    Second pure Python probing code is tried.
+    Second, pure Python probing code is tried.
     """
     try:
         if LibraryLoader:
@@ -2105,10 +2106,3 @@ def open_tablebase(directory, *, libgtb=None, LibraryLoader=ctypes.cdll):
     tables = PythonTablebase()
     tables.add_directory(directory)
     return tables
-
-
-# TODO: Deprecated
-open_tablebases_native = open_tablebase_native
-open_tablebases = open_tablebase
-PythonTablebases = PythonTablebase
-NativeTablebases = NativeTablebase
