@@ -1,7 +1,7 @@
 # -*- coding: utf-8 -*-
 #
 # This file is part of the python-chess library.
-# Copyright (C) 2012-2018 Niklas Fiekas <niklas.fiekas@backscattering.de>
+# Copyright (C) 2012-2019 Niklas Fiekas <niklas.fiekas@backscattering.de>
 #
 # This program is free software: you can redistribute it and/or modify
 # it under the terms of the GNU General Public License as published by
@@ -102,6 +102,7 @@ MOVETEXT_REGEX = re.compile(r"""
     |([\?!]{1,2})
     """, re.DOTALL | re.VERBOSE)
 
+SKIP_MOVETEXT_REGEX = re.compile(r""";|\{|\}""")
 
 TAG_ROSTER = ["Event", "Site", "Date", "Round", "White", "Black", "Result"]
 
@@ -208,9 +209,6 @@ class GameNode:
 
         return True
 
-    # TODO: Deprecated
-    is_main_line = is_mainline
-
     def is_main_variation(self):
         """
         Checks if this node is the first variation from the point of view of its
@@ -294,9 +292,6 @@ class GameNode:
         """Returns an iterator over the main moves after this node."""
         return Mainline(self, lambda node: node.move)
 
-    # TODO: Deprecated
-    main_line = mainline_moves
-
     def add_line(self, moves, *, comment="", starting_comment="", nags=()):
         """
         Creates a sequence of child nodes for the given list of moves.
@@ -325,6 +320,10 @@ class GameNode:
 
         visitor.visit_move(parent_board, self.move)
 
+        parent_board.push(self.move)
+        visitor.visit_board(parent_board)
+        parent_board.pop()
+
         for nag in sorted(self.nags):
             visitor.visit_nag(nag)
 
@@ -379,6 +378,8 @@ class GameNode:
                 visitor.visit_header(tagname, tagvalue)
 
             if visitor.end_headers() is not SKIP:
+                visitor.visit_board(board)
+
                 if self.variations:
                     self.variations[0].accept(visitor, _parent_board=board)
 
@@ -455,11 +456,14 @@ class Game(GameNode):
             for tagname, tagvalue in self.headers.items():
                 visitor.visit_header(tagname, tagvalue)
             if visitor.end_headers() is not SKIP:
+                board = self.board()
+                visitor.visit_board(board)
+
                 if self.comment:
                     visitor.visit_comment(self.comment)
 
                 if self.variations:
-                    self.variations[0].accept(visitor, _parent_board=self.board())
+                    self.variations[0].accept(visitor, _parent_board=board)
 
                 visitor.visit_result(self.headers.get("Result", "*"))
 
@@ -697,6 +701,13 @@ class BaseVisitor:
         """
         pass
 
+    def visit_board(self, board):
+        """
+        Called for the starting position of the game and after each move.
+
+        The board state must be restored before the traversal continues.
+        """
+
     def visit_comment(self, comment):
         """Called for each comment."""
         pass
@@ -735,7 +746,7 @@ class BaseVisitor:
         raise error
 
 
-class GameModelCreator(BaseVisitor):
+class GameCreator(BaseVisitor):
     """
     Creates a game model. Default visitor for :func:`~chess.pgn.read_game()`.
     """
@@ -747,6 +758,9 @@ class GameModelCreator(BaseVisitor):
         self.starting_comment = ""
         self.in_variation = False
 
+    def begin_headers(self):
+        return self.game.headers
+
     def visit_header(self, tagname, tagvalue):
         self.game.headers[tagname] = tagvalue
 
@@ -802,6 +816,7 @@ class HeaderCreator(BaseVisitor):
 
     def begin_headers(self):
         self.headers = Headers({})
+        return self.headers
 
     def visit_header(self, tagname, tagvalue):
         self.headers[tagname] = tagvalue
@@ -813,6 +828,30 @@ class HeaderCreator(BaseVisitor):
         return self.headers
 
 
+class BoardCreator(BaseVisitor):
+    """
+    Returns the final position of the game. The mainline of the game is
+    on the move stack.
+    """
+
+    def begin_game(self):
+        self.skip_variation_depth = 0
+
+    def begin_variation(self):
+        self.skip_variation_depth += 1
+        return SKIP
+
+    def end_variation(self):
+        self.skip_variation_depth = max(self.skip_variation_depth - 1, 0)
+
+    def visit_board(self, board):
+        if not self.skip_variation_depth:
+            self.board = board
+
+    def result(self):
+        return self.board
+
+
 class SkipVisitor(BaseVisitor):
     """Skips a game."""
 
@@ -980,7 +1019,7 @@ class FileExporter(StringExporter):
         return self.__repr__()
 
 
-def read_game(handle, *, Visitor=GameModelCreator):
+def read_game(handle, *, Visitor=GameCreator):
     """
     Reads a game from a file opened in text mode.
 
@@ -1004,7 +1043,7 @@ def read_game(handle, *, Visitor=GameModelCreator):
 
     By using text mode, the parser does not need to handle encodings. It is the
     caller's responsibility to open the file with the correct encoding.
-    PGN files are ASCII or UTF-8 most of the time. So, the following should
+    PGN files are usually ASCII or UTF-8 encoded. So, the following should
     cover most relevant cases (ASCII, UTF-8, UTF-8 with BOM).
 
     >>> pgn = open("data/pgn/kasparov-deep-blue-1997.pgn", encoding="utf-8-sig")
@@ -1017,7 +1056,7 @@ def read_game(handle, *, Visitor=GameModelCreator):
     >>> game = chess.pgn.read_game(pgn)
 
     The end of a game is determined by a completely blank line or the end of
-    the file. (Of course, blank lines in comments are possible.)
+    the file. (Of course, blank lines in comments are possible).
 
     According to the PGN standard, at least the usual 7 header tags are
     required for a valid game. This parser also handles games without any
@@ -1026,7 +1065,7 @@ def read_game(handle, *, Visitor=GameModelCreator):
     The parser is relatively forgiving when it comes to errors. It skips over
     tokens it can not parse. Any exceptions are logged and collected in
     :data:`Game.errors <chess.pgn.Game.errors>`. This behavior can be
-    :func:`overriden <chess.pgn.GameModelCreator.handle_error>`.
+    :func:`overriden <chess.pgn.GameCreator.handle_error>`.
 
     Returns the parsed game or ``None`` if the end of file is reached.
     """
@@ -1035,22 +1074,29 @@ def read_game(handle, *, Visitor=GameModelCreator):
     found_game = False
     skipping_game = False
     headers = None
+    managed_headers = None
 
-    # Skip leading empty lines and comments.
+    # Ignore leading empty lines and comments.
     line = handle.readline()
     while line.isspace() or line.startswith("%") or line.startswith(";"):
         line = handle.readline()
 
     # Parse game headers.
     while line:
-        # Skip comments.
+        # Ignore comments.
         if line.startswith("%") or line.startswith(";"):
             line = handle.readline()
             continue
 
+        # First token of the game.
         if not found_game:
             found_game = True
             skipping_game = visitor.begin_game() is SKIP
+            if not skipping_game:
+                managed_headers = visitor.begin_headers()
+                if not isinstance(managed_headers, Headers):
+                    managed_headers = None
+                    headers = Headers({})
 
         if not line.startswith("["):
             break
@@ -1058,12 +1104,9 @@ def read_game(handle, *, Visitor=GameModelCreator):
         if not skipping_game:
             tag_match = TAG_REGEX.match(line)
             if tag_match:
-                if headers is None:
-                    headers = Headers({})
-                    visitor.begin_headers()
-
-                headers[tag_match.group(1)] = tag_match.group(2)
                 visitor.visit_header(tag_match.group(1), tag_match.group(2))
+                if headers is not None:
+                    headers[tag_match.group(1)] = tag_match.group(2)
             else:
                 break
 
@@ -1072,10 +1115,10 @@ def read_game(handle, *, Visitor=GameModelCreator):
     if not found_game:
         return None
 
-    if headers is not None:
+    if not skipping_game:
         skipping_game = visitor.end_headers() is SKIP
 
-    # Skip a single empty line after headers.
+    # Ignore single empty line after headers.
     if line.isspace():
         line = handle.readline()
 
@@ -1084,40 +1127,51 @@ def read_game(handle, *, Visitor=GameModelCreator):
         in_comment = False
 
         while line:
-            if not in_comment and line.isspace():
-                break
-            elif (not in_comment and "{" in line) or (in_comment and "}" in line):
-                in_comment = line.rfind("{") > line.rfind("}")
+            if not in_comment:
+                if line.isspace():
+                    break
+                elif line.startswith("%"):
+                    line = handle.readline()
+                    continue
+
+            for match in SKIP_MOVETEXT_REGEX.finditer(line):
+                token = match.group(0)
+                if token == "{":
+                    in_comment = True
+                elif not in_comment and token == ";":
+                    break
+                elif token == "}":
+                    in_comment = False
 
             line = handle.readline()
 
         visitor.end_game()
         return visitor.result()
 
-    # Chess variant and initial position.
-    if headers is None:
-        headers = Headers({})
-
+    # Chess variant.
+    headers = managed_headers if headers is None else headers
     try:
         VariantBoard = headers.variant()
     except ValueError as error:
         visitor.handle_error(error)
         VariantBoard = chess.Board
 
+    # Initial position.
     fen = headers.get("FEN", VariantBoard.starting_fen)
     try:
         board_stack = [VariantBoard(fen, chess960=headers.is_chess960())]
     except ValueError as error:
         visitor.handle_error(error)
         board_stack = [VariantBoard(chess960=headers.is_chess960())]
+    visitor.visit_board(board_stack[0])
 
     # Parse movetext.
     skip_variation_depth = 0
     while line:
         read_next_line = True
 
+        # Ignore comments.
         if line.startswith("%") or line.startswith(";"):
-            # Ignore comments.
             line = handle.readline()
             continue
 
@@ -1202,6 +1256,7 @@ def read_game(handle, *, Visitor=GameModelCreator):
                 else:
                     visitor.visit_move(board_stack[-1], move)
                     board_stack[-1].push(move)
+                visitor.visit_board(board_stack[-1])
 
         if read_next_line:
             line = handle.readline()
@@ -1256,3 +1311,7 @@ def skip_game(handle):
     Skip a game. Returns ``True`` if a game was found and skipped.
     """
     return read_game(handle, Visitor=SkipVisitor)
+
+
+# TODO: Deprecated
+GameModelCreator = GameCreator
