@@ -1,7 +1,7 @@
 # -*- coding: utf-8 -*-
 #
 # This file is part of the python-chess library.
-# Copyright (C) 2012-2018 Niklas Fiekas <niklas.fiekas@backscattering.de>
+# Copyright (C) 2012-2019 Niklas Fiekas <niklas.fiekas@backscattering.de>
 #
 # This program is free software: you can redistribute it and/or modify
 # it under the terms of the GNU General Public License as published by
@@ -1483,11 +1483,14 @@ class Tablebase:
 
     def add_directory(self, directory, *, load_wdl=True, load_dtz=True):
         """
-        Loads tables from a directory.
+        Adds tables from a directory.
 
         By default all available tables with the correct file names
         (e.g. WDL files like ``KQvKN.rtbw`` and DTZ files like ``KRBvK.rtbz``)
-        are loaded.
+        are added.
+
+        The relevant files are lazily opened when the tablebase is actually
+        probed.
 
         Returns the number of table files that were found.
         """
@@ -1513,9 +1516,6 @@ class Tablebase:
 
         return num
 
-    # TODO: Deprecated
-    open_directory = add_directory
-
     def probe_wdl_table(self, board):
         # Test for variant end.
         if board.is_variant_win():
@@ -1651,6 +1651,8 @@ class Tablebase:
             be found in the tablebase. Use
             :func:`~chess.syzygy.Tablebase.get_wdl()` if you prefer to get
             ``None`` instead of an exception.
+
+            Note that probing corrupted table files is undefined behavior.
         """
         # Positions with castling rights are not in the tablebase.
         if board.castling_rights:
@@ -1846,6 +1848,8 @@ class Tablebase:
             be found in the tablebase. Use
             :func:`~chess.syzygy.Tablebase.get_dtz()` if you prefer to get
             ``None`` instead of an exception.
+
+            Note that probing corrupted table files is undefined behavior.
         """
         v = self.probe_dtz_no_ep(board)
 
@@ -1930,7 +1934,3 @@ def open_tablebase(directory, *, load_wdl=True, load_dtz=True, max_fds=128, Vari
     tables = Tablebase(max_fds=max_fds, VariantBoard=VariantBoard)
     tables.add_directory(directory, load_wdl=load_wdl, load_dtz=load_dtz)
     return tables
-
-# TODO: Deprecated
-open_tablebases = open_tablebase
-Tablebases = Tablebase
